#!/bin/sh

# Script for IMSA 7 patching via Web Console.
# inherited from IMSA 1 apply.sh
# ini_util.sh is based on StarGate implementation, with minor bug fixes.
# Zhennan, Nov/07/2006
#
# This script is invoked by apply.sh, which is invoked by imssmgr

export PATCH_FOLDER=${IMSS_HOME}/patch
export TEMP_FOLDER=${PATCH_FOLDER}/temp

TEMP_BINARY_TAR_GZ=${TEMP_FOLDER}/binary.tar.gz
TEMP_EN_SPECIFIC_DIR=${TEMP_FOLDER}/en_specific
TEMP_JA_SPECIFIC_DIR=${TEMP_FOLDER}/jp_specific
TEMP_ROLLBACK_SH=${TEMP_FOLDER}/rollback.sh
TEMP_INI_UTIL_SH=${TEMP_FOLDER}/ini_util.sh
TEMP_PATCH_COMMON_SH=${TEMP_FOLDER}/patch_common.sh
TEMP_BACKUP_TAR=${TEMP_FOLDER}/backup.tar
TEMP_BACKUP_TAR_GZ=${TEMP_FOLDER}/backup.tar.gz
TEMP_BACKUP_FILELIST=${TEMP_FOLDER}/backup.filelist
APP_TEMP_IN_HOME=${IMSS_HOME}/app_temp
APP_TEMP_IN_APP_DATA=/var/app_data/imss/app_temp/
#EULA is not used by scripts.

NORMAL_ROLLBACK_SH=${PATCH_FOLDER}/rollback.sh
NORMAL_INI_UTIL_SH=${PATCH_FOLDER}/ini_util.sh
NORMAL_PATCH_COMMON_SH=${PATCH_FOLDER}/patch_common.sh
NORMAL_BACKUP_TAR_GZ=${PATCH_FOLDER}/backup.tar.gz
NORMAL_BACKUP_FILELIST=${PATCH_FOLDER}/backup.filelist


TEMP_APPLY_DB_SH=${TEMP_FOLDER}/apply_db.sh
TEMP_APPLY_OS_SH=${TEMP_FOLDER}/apply_os.sh
TEMP_APPLY_SPECIAL_SH=${TEMP_FOLDER}/apply_special.sh
TOMCAT_CACHE_FOLDER=${IMSS_HOME}/UI/adminUI/ROOT/work/
EUQ_CACHE_FOLDER=${IMSS_HOME}/UI/euqUI/ROOT/work/

. ${TEMP_INI_UTIL_SH}
. ${TEMP_PATCH_COMMON_SH}

PATCH_NUMBER=""
ERROR_NUMBER=0

NO_BACKUP_FILE=0
export UpdateDLPComplianceTemplateFullXML=0
PARAMETER_UNREG="notunreg"

# PATCHINFOINI is exported in apply.sh
# but to ease the testing, define it here again
export PATCHINFOINI=${TEMP_FOLDER}/patchinfo.ini
RESULT_BUILD=`F_ini_get ${PATCHINFOINI} "General" "BuildNum"`
PATCH_TITLE=`F_ini_get ${PATCHINFOINI} "General" "Title"`

PATCH_ROLLBACKABLE=`F_ini_get $PATCHINFOINI "Application" "Rollbackable"`
if [ -z "$PATCH_ROLLBACKABLE" ] ; then
    PATCH_ROLLBACKABLE="0"
fi

UI_INTERRUPTION=`F_ini_get $PATCHINFOINI "General" "UIInterruption"`
if [ -z "$UI_INTERRUPTION" ] ; then
    UI_INTERRUPTION="0"
fi

IS_MASTER=0
sh /opt/trend/imss/script/is_master.sh
if [ $? = 1 ] ; then
    IS_MASTER=1
fi

F_check_required_files_and_fields() {

    if [ ! -f "$PATCHMGMINI" ] ; then
        touch $PATCHMGMINI
        if [ $? -ne 0 ] ; then
            F_log "Can not create $PATCHMGMINI."
            ERROR_NUMBER=${ERR_INI_FILE_ACCESS}
            return 1
        fi
    fi

    if [ ! -f "$TEMP_BINARY_TAR_GZ" ] ; then
        F_log "Can not find $TEMP_BINARY_TAR_GZ."
        ERROR_NUMBER=${ERR_FILE_MISSING}
        return 1
    fi
    
    if [ ! -d "$TEMP_EN_SPECIFIC_DIR" ] ; then
        F_log "Can not find $TEMP_EN_SPECIFIC_DIR."
        ERROR_NUMBER=${ERR_FILE_MISSING}
        return 1
    fi 

    if [ ! -d "$TEMP_JA_SPECIFIC_DIR" ] ; then
        F_log "Can not find $TEMP_JA_SPECIFIC_DIR."
        ERROR_NUMBER=${ERR_FILE_MISSING}
        return 1
    fi       

    if [ $PATCH_ROLLBACKABLE = "1" ] ; then
        if [ ! -f "$TEMP_ROLLBACK_SH" ] ; then
            F_log "Can not find $TEMP_ROLLBACK_SH."
            ERROR_NUMBER=${ERR_FILE_MISSING}
            return 1
        fi
    fi

    # Number, MD5, RequireDiskSpace are required

    PATCH_NUMBER=`F_ini_get $PATCHINFOINI Application Number`
    if [ -z "$PATCH_NUMBER" ] ; then
        F_log "Cannot find Number in $PATCHINFOINI."
        ERROR_NUMBER=${ERR_INI_FILE_CONTENT}
        return 1
    fi

    _value=`F_ini_get $PATCHINFOINI Application MD5`
    if [ -z "$_value" ] ; then
        F_log "Cannot find MD5 in $PATCHINFOINI."
        ERROR_NUMBER=${ERR_INI_FILE_CONTENT}
        return 1
    fi

    _value=`F_ini_get $PATCHINFOINI Application RequireDiskSpace`
    if [ -z "$_value" ] ; then
        F_log "Cannot find RequireDiskSpace in $PATCHINFOINI."
        ERROR_NUMBER=${ERR_INI_FILE_CONTENT}
        return 1
    fi

    F_log "Required file and fields for application update check OK."

    return 0
}

F_check_redundancy() {

    _applied_patches=`F_ini_get $PATCHMGMINI Patches Applied`

    #stop if the patch number is already in Applied..
    for i in $_applied_patches
    do
        if [ "$PATCH_NUMBER" = "$i" ] ; then
            F_log "Cannot apply this patch. The patch script has detected a same patch number."
            ERROR_NUMBER=${ERR_ALREADY_APPLIED}
            return 1
        fi
    done

    #delete the section if there is an old section
    F_ini_get_section $PATCHMGMINI $PATCH_NUMBER
    if [ $? -eq 0 ] ; then
        F_log "The patch section is still in $PATCHMGMINI. The old section will be overwritten."
        F_ini_delete_section $PATCHMGMINI $PATCH_NUMBER
    fi

    F_log "Redundancy Check OK."
    return 0
}

F_check_dependency() {
    #Return error if I require a patch that doesn't exist.

    _dependency_number=`F_ini_get $PATCHINFOINI Application Dependency`
    if [ -z "$_dependency_number" ] ; then
        F_log "No dependency requirement."
        return 0
    fi
    if [ -n "$_dependency_number" ] ; then

        _found_require=0
        for j in $_applied_patches
        do
            if [ "$_dependency_number" = "$j" ] ; then
                _found_require=1
            fi
        done
        if [ "$_found_require" = "0" ] ; then
            F_log "Cannot find the required previous dependency patch $_dependency_number."
            ERROR_NUMBER=${ERR_DEPENDENCY_MISSING}
            return 1
        fi
        F_log "Find the required previous dependency patch $_dependency_number."
    fi

    return 0
}

F_check_free_disk() {

    _available_disk_size=`df -P -m /opt/trend | tail -1 | awk '{print $4}'`
    _required_disk_size=`F_ini_get $PATCHINFOINI Application RequireDiskSpace`
    _sub_of_disk_size=`expr ${_available_disk_size} - ${_required_disk_size}`
    if [ ${_sub_of_disk_size} -gt 0 ] ; then
        F_log "Disk space check passed: ${_available_disk_size} > ${_required_disk_size}"
        return 0
    else
        F_log "Not enough disk space: ${_available_disk_size} <= ${_required_disk_size}"
        ERROR_NUMBER=${ERR_DISK_SPACE}
        return 1
    fi
    return 0
}

F_check_md5sum() {

    _real_md5sum=`md5sum $TEMP_BINARY_TAR_GZ | awk '{print $1}'`
    _said_md5sum=`F_ini_get $PATCHINFOINI Application MD5`
    if [ ${_real_md5sum} = ${_said_md5sum} ] ; then
        F_log "md5sum check passed: ${_real_md5sum}"
        return 0
    else
        F_log "md5sum check fail. real:${_real_md5sum} expected:${_said_md5sum}"
        ERROR_NUMBER=${ERR_MD5SUM}
        return 1
    fi

    return 0
}


F_check_all() {

    F_check_required_files_and_fields
    if [ $? -ne 0 ] ; then
        F_log "The patch format is incorrect. Please check required files or fields in the configuration file."
        return 1
    fi

    F_check_redundancy
    if [ $? -ne 0 ] ; then
        F_log "Cannot pass redundancy check. Exit."
        return 1
    fi

    F_check_dependency
    if [ $? -ne 0 ] ; then
        F_log "Cannot pass dependency check. Exit."
        return 1
    fi

    F_check_free_disk
    if [ $? -ne 0 ] ; then
        F_log "Cannot pass free disk check. Exit."
        return 1
    fi

    F_check_md5sum
    if [ $? -ne 0 ] ; then
        F_log "Cannot pass md5 check. Exit."
        return 1
    fi

}

F_backup() {

    # generate backup.filelist first
    tar -ztvf ${TEMP_BINARY_TAR_GZ} | grep -v "/$" | awk '{print $6}' > ${TEMP_BACKUP_FILELIST}
    if [ $? -ne 0 ] ; then
        F_log "Generate backup.filelist failed."
        ERROR_NUMBER=${ERR_BACKUP}
        return 1
    fi

    # backup these files, some maybe missing from current system, so, ignore-failed-read

    #Add by Marvin 2006-2-15 22:56
    len=`wc -l < $TEMP_BACKUP_FILELIST`
    done_file_cnt=0

    for i in `seq 1 $len`; do
        f=`head -n $i $TEMP_BACKUP_FILELIST | tail -n 1`
        if [ -r /$f ]; then
            done_file_cnt=`expr ${done_file_cnt} + 1`
            if [ $i -eq 0 ]; then
                tar cf $TEMP_BACKUP_TAR -C / $f
                if [ $? -ne 0 ] ; then
                    F_log "Generate backup.tar failed."
                    ERROR_NUMBER=${ERR_BACKUP}
                    return 1
                fi
            else
                tar rf $TEMP_BACKUP_TAR -C / $f
                if [ $? -ne 0 ] ; then
                    F_log "Append backup.tar failed."
                    ERROR_NUMBER=${ERR_BACKUP}
                    return 1
                fi
            fi
        fi
    done

    rm -f $TEMP_BACKUP_TAR_GZ

    if [ ${done_file_cnt} -ne 0 ] ; then
        gzip $TEMP_BACKUP_TAR
        if [ $? -ne 0 ] ; then
            F_log "Generate backup.tar.gz failed."
            ERROR_NUMBER=${ERR_BACKUP}
            return 1
        fi
    else
        NO_BACKUP_FILE=1
    fi

    #End by Marvin
    F_log "Backup done."
    return 0;
}

F_apply_now() {

    tar -C / -zxf ${TEMP_BINARY_TAR_GZ}
    if [ $? -ne 0 ] ; then
        F_log "Can not untar patch files. Auto rollback."
        ERROR_NUMBER=${ERR_UNTAR_PATCH_FILES}

        # auto rollback
        pushd .
        cd /
        rm -rf `cat ${TEMP_BACKUP_FILELIST}`
        popd
        tar -C / -zxf ${TEMP_BACKUP_TAR_GZ}

        return 1
    fi

    F_log "Apply patch files done."
    return 0;
}

F_post_apply() {

    # update patch.ini

    F_ini_exist_section ${PATCHMGMINI} "Patches"
    if [ $? -ne 0 ] ; then
        F_ini_append_section ${PATCHMGMINI} "Patches"
    fi

    _value=`F_ini_get ${PATCHMGMINI} "Patches" "Applied"`
    F_ini_add_key ${PATCHMGMINI} "Patches" "Applied" "$_value $PATCH_NUMBER"

    F_ini_add_key ${PATCHMGMINI} "Patches" "Rollbackable" "$PATCH_NUMBER"

    F_ini_exist_section ${PATCHMGMINI} "$PATCH_NUMBER"
    if [ $? -ne 1 ] ; then
        F_ini_delete_section ${PATCHMGMINI} "$PATCH_NUMBER"
    fi

    F_ini_append_section ${PATCHMGMINI} "$PATCH_NUMBER"

    _value=`F_ini_get ${PATCHINFOINI} "General" "BuildNum"`
    F_ini_add_key ${PATCHMGMINI} "$PATCH_NUMBER" "BuildNum" "${_value}"
    RESULT_BUILD=${_value}

    _value=`F_ini_get ${PATCHINFOINI} "General" "Title"`
    F_ini_add_key ${PATCHMGMINI} "$PATCH_NUMBER" "Title" "${_value}"
    PATCH_TITLE=${_value}

    _value=`F_ini_get ${PATCHINFOINI} "General" "Description"`
    F_ini_add_key ${PATCHMGMINI} "$PATCH_NUMBER" "Description" "${_value}"

    _value=`F_ini_get ${PATCHINFOINI} "Application" "CmdBeforeUpdate"`
    F_ini_add_key ${PATCHMGMINI} "$PATCH_NUMBER" "CmdBeforeUpdate" "${_value}"

    _value=`F_ini_get ${PATCHINFOINI} "Application" "CmdAfterUpdate"`
    F_ini_add_key ${PATCHMGMINI} "$PATCH_NUMBER" "CmdAfterUpdate" "${_value}"

    F_ini_add_key ${PATCHMGMINI} "$PATCH_NUMBER" "NoBackupFile" "${NO_BACKUP_FILE}"

    F_ini_add_key ${PATCHMGMINI} "$PATCH_NUMBER" "UIInterruption" "${UI_INTERRUPTION}"

    # backup files

    cp ${TEMP_ROLLBACK_SH} ${NORMAL_ROLLBACK_SH}
    cp ${TEMP_INI_UTIL_SH} ${NORMAL_INI_UTIL_SH}
    cp ${TEMP_PATCH_COMMON_SH} ${NORMAL_PATCH_COMMON_SH}
    rm -f ${NORMAL_BACKUP_TAR_GZ}
    mv ${TEMP_BACKUP_TAR_GZ} ${NORMAL_BACKUP_TAR_GZ} 2>/dev/null
    mv ${TEMP_BACKUP_FILELIST} ${NORMAL_BACKUP_FILELIST}

    return 0
}


F_apply() {

    F_backup
    if [ $? -ne 0 ] ; then
        F_log "Cannot backup existing files. Exit."
        return 1
    fi

    if [ $UI_INTERRUPTION = "1" ] ; then
    if [ $IS_MASTER = 1 ] ; then
        /opt/trend/imss/PostgreSQL/bin/psql -U sa -d imss -c "update tb_patch_status set status=4 where status=2;" > /dev/null 2>&1
        sleep 10
        /opt/trend/imss/PostgreSQL/bin/psql -U sa -d imss -c "update tb_patch_status set status=2 where status=4;" > /dev/null 2>&1
    fi
    fi

    if [ $IS_MASTER -eq 1 ] ; then
        F_log "Apply Database update."
        . ${TEMP_APPLY_DB_SH} "install" ${PATCH_FOLDER} ${PATCH_NUMBER}
        
        . ${TEMP_APPLY_DB_SH} "updatedbschema" ${PATCH_FOLDER} ${PATCH_NUMBER}
        if [ $? -ne 0 ];then
        
            F_log "Cannot update database, stop apply hot fix."
            return 1
        fi
    fi
    
    
    _value=`F_ini_get ${PATCHINFOINI} "Application" "CmdBeforeUpdate"`
    if [ ! -z "$_value" ] ; then
        F_log "Calling ${_value}"
        sh -c "${_value}"
    fi

    # Clear tomcat cache folder, just in case.
    if [  -d "$TOMCAT_CACHE_FOLDER" ] ; then
        echo "clean adminUI cache data"
        rm -rf ${TOMCAT_CACHE_FOLDER}
    fi
    if [  -d "$EUQ_CACHE_FOLDER" ] ; then 
        echo "clean euqUI cache data"
        rm -rf ${EUQ_CACHE_FOLDER}
    fi
    #SEGTT-325689
    if [ ! -d /var/app_data/imss/app_temp/temp ];then
        echo "$APP_TEMP_IN_HOME/temp doesn't exists, Create it"
        cp -pfr /opt/trend/imss/temp /var/app_data/imss/app_temp/temp.hfbac 
        mv /opt/trend/imss/temp /var/app_data/imss/app_temp/.
        #create soft link , why need rm , because au may create temp folder  again  
        rm -rf /opt/trend/imss/temp && ln -s /var/app_data/imss/app_temp/temp /opt/trend/imss/temp
    fi
	 #SEGTT-325689-END
    #SEGTT-311002
    . ${TEMP_APPLY_SPECIAL_SH}   "install"
    #SEGTT-glibc
    . ${TEMP_APPLY_OS_SH}   "install"
    
    #[SEGTT-311427]
    if [ ! -h "/lib/libcom_err.so.3" ]; then
        ln -s /opt/trend/imss/lib/libcom_err.so /lib/libcom_err.so.3
        F_log "Create link /lib/libcom_err.so.3 ."
    fi
    F_apply_now
    if [ $? -ne 0 ] ; then
        _value=`F_ini_get ${PATCHINFOINI} "Application" "CmdAfterUpdate"`
        if [ ! -z "$_value" ] ; then
            F_log "Calling ${_value}"
            sh -c "${_value}"
        fi
        F_log "Cannot apply new patch files. Exit."
        return 1
    fi

    F_post_apply
    # no error check for F_post_apply
     
    # must write result before call CmdAfterUpdate
    # otherwise, imssmgr may not able to find the result during startup
    F_write_result "1" "1" "$PATCH_ROLLBACKABLE" "0" "${ERR_MSG_SUCCESS}" "${RESULT_BUILD}" "${PATCH_TITLE}"

    _value=`F_ini_get ${PATCHINFOINI} "Application" "CmdAfterUpdate"`
    if [ ! -z "$_value" ] ; then
        F_log "Calling ${_value}"
        sh -c "${_value}"
    fi

    if [ ${UpdateDLPComplianceTemplateFullXML} -eq 1 ]; then
        sh -c /opt/trend/imss/script/UpdateDlpTemplate.sh
    fi
    return 0
}

F_main() {

    F_check_all

    if [ $? -ne 0 ] ; then
        return 1
    fi

    F_gen_lan_package

    if [ $? -ne 0 ] ; then
        return 1
    fi
    
    F_apply

    if [ $? -ne 0 ] ; then
        return 1
    fi


    # Success
    F_log "Done. Success"
    return 0
}

F_save_self() {

    if [ ${PARAMETER_UNREG} = "unreg" ] ; then
        return 0
    fi

    mkdir /mnt/backup/patches > /dev/null 2>&1
    
    # use patch number as filename
    # and tar all files in /opt/trend/patch/temp/*
    cd ${TEMP_FOLDER}
    tar zcf /mnt/backup/patches/${PATCH_NUMBER} * > /dev/null 2>&1
    
    if [ $? -ne 0 ] ; then
        F_log "Cannot create application patch package in image partition."
        return 1
    else
        echo ${PATCH_NUMBER} >> /mnt/backup/patches/patches.history
    fi
    
    return 0
}

F_gen_lan_package() {

BINARY_NAME="binary"
BINARY_TAR="${TEMP_FOLDER}/${BINARY_NAME}.tar"
BINARY_TAR_GZ="${BINARY_TAR}.gz"

GZIP="/bin/gzip" 
TAR="/bin/tar" 
FIND="/usr/bin/find" 
XARGS="/usr/bin/xargs" 
TOUCH="/bin/touch" 

#Get the current IMSS locale
IMSS_INI_DB=${IMSS_HOME}/config/imss.ini.db
IS_LOCALE_EN=`grep locale ${IMSS_INI_DB} | grep ja >/dev/null 2>&1 ; echo $?` 

if [  ${IS_LOCALE_EN} -eq 1 ];then 
	F_log "product locale: en"
	BIN_SRC_DIR=${TEMP_EN_SPECIFIC_DIR} 
else
	F_log "product locale: ja"
	BIN_SRC_DIR="${TEMP_JA_SPECIFIC_DIR}" 
fi

# check if there's language specific binaries need to be updatd
exist_files=`${FIND} ${BIN_SRC_DIR} -type f  -name "*" `
if [ "${exist_files}" == "" ];then
	F_log "no language specific files, skip to next steps ..."
	return 0
fi

# make sure the binaries are newer
`${FIND} ${BIN_SRC_DIR} -name "*" | ${XARGS} ${TOUCH}`

# extract the original package to get the tar file
msg= `${GZIP} -d ${BINARY_TAR_GZ} 2>&1`
ret=$?

if [ ${ret} -ne 0 ]; then
	F_log "extract file fail: [${msg}]"
	return -1
fi

#apply the files
CWD=`pwd`
pushd . > /dev/null
cd ${BIN_SRC_DIR} 
msg=`${TAR} -u --file="${BINARY_TAR}" ./ 2>&1`
ret=$?
popd > /dev/null

if [ ${ret} -ne 0 ];then
	F_log "update binary failed: [${msg}]"
	return -1 
fi

# package the file again

msg=`${GZIP} ${BINARY_TAR} 2>&1`
ret=$?

if [ ${ret} -ne 0 ];then
	F_log "packaging binary failed: [${msg}]"
	return -1 
fi

return 0

}

# IMSA-7.0-00742 
if [ ! -z $1 ] ; then
    if [ $1 = "unreg" ] ; then
        PARAMETER_UNREG="unreg"
    fi
fi

F_main
if [ $? -ne 0 ] ; then
    # result_build and title doesn't matter, they won't be used when fail
    ## it matters because the stupid ConfigFileOnDisck class doesn't OK if any key has no value
    F_write_result "1" "1" "0" "1" "${ERR_MSG_FAIL}" "${RESULT_BUILD}" "${PATCH_TITLE}"
    exit ${ERROR_NUMBER}
else
    # save app patch in hidden partition for rescure
    # do not report fail just because save fail.
    F_save_self

    exit 0
fi
