#!/bin/sh

# Script for IMSA patch rollbacking via Web Console.
# ini_util.sh is based on StarGate implementation, with minor bug fixes.
# Zhennan, Nov/07/2006
#
# This script is invoked by imssmgr, which is running as root
# when invoked, stdout and stderr will be redirected to log file: patchlog.txt
# for easy testing, no stdout and stderr redirection inside the script.
# this script can be executed from console directly, with correct folder structure
# 
# Naming convention
# F_  means a subroutine

# !!! Comment this before check-in !!! It's just for testing
# export IMSS_HOME=/home/szn/fakeimss

#PATCH_FOLDER=${IMSS_HOME}/patch
export PATCH_FOLDER=${IMSS_HOME}/patch
export TEMP_FOLDER=${PATCH_FOLDER}/temp
PATCHMGMINI=${PATCH_FOLDER}/patch.ini
LOCK_FILE=${PATCH_FOLDER}/.isinprogress

NORMAL_ROLLBACK_SH=${PATCH_FOLDER}/rollback.sh
NORMAL_INI_UTIL_SH=${PATCH_FOLDER}/ini_util.sh
NORMAL_PATCH_COMMON_SH=${PATCH_FOLDER}/patch_common.sh
NORMAL_BACKUP_TAR_GZ=${PATCH_FOLDER}/backup.tar.gz
NORMAL_BACKUP_FILELIST=${PATCH_FOLDER}/backup.filelist
TEMP_CRONTAB=${TEMP_FOLDER}/cron.temp
TAG_DELETE_VACUUM_CRON=${PATCH_FOLDER}/tag_delete_vacuum_cron
TEMP_APPLY_DB_SH=$PATCH_FOLDER/temp/apply_db.sh

. ${NORMAL_INI_UTIL_SH}
. ${NORMAL_PATCH_COMMON_SH}

PATCH_NUMBER=""
ERROR_NUMBER=0
NO_BACKUP_FILE=0

F_check_all() {
    if [ ! -f "$PATCHMGMINI" ] ; then
        F_log "Can not find $PATCHMGMINI."
        ERROR_NUMBER=${ERR_INI_FILE_ACCESS}
        return 1
    fi
    
    if [ ! -f "$NORMAL_BACKUP_FILELIST" ] ; then
        F_log "Can not find $NORMAL_BACKUP_FILELIST."
        ERROR_NUMBER=${ERR_FILE_MISSING}
        return 1
    fi

    # key, section, are required
    
    PATCH_NUMBER=`F_ini_get $PATCHMGMINI Patches Rollbackable`
    if [ -z "$PATCH_NUMBER" ] ; then
        F_log "Cannot find rollbackable patch Number in $PATCHMGMINI."
        ERROR_NUMBER=${ERR_INI_FILE_CONTENT}    
        return 1
    fi
    
    #stop if the patch number can not be found in Applied
    _applied_patches=`F_ini_get $PATCHMGMINI Patches Applied`
    _found_in_applied=""
    for i in $_applied_patches 
    do
        if [ "$PATCH_NUMBER" = "$i" ] ; then
            _found_in_applied="found"
        fi
    done
    if [ -z ${_found_in_applied} ] ; then
        F_log "Cannot find rollbackable patch Number in Applied patches."
        ERROR_NUMBER=${ERR_INI_FILE_CONTENT}    
        return 1
    fi

    F_ini_exist_section ${PATCHMGMINI} "$PATCH_NUMBER"
    if [ $? -ne 0 ] ; then
        F_log "Cannot find rollback info for patch $PATCH_NUMBER."
        ERROR_NUMBER=${ERR_INI_FILE_CONTENT}    
        return 1
    fi

    NO_BACKUP_FILE=`F_ini_get $PATCHMGMINI "$PATCH_NUMBER" "NoBackupFile"`
    if [ $NO_BACKUP_FILE -eq 0 ] ; then
        if [ ! -f "$NORMAL_BACKUP_TAR_GZ" ] ; then
            F_log "Can not find $NORMAL_BACKUP_TAR_GZ."
            ERROR_NUMBER=${ERR_FILE_MISSING}
            return 1
        fi
    fi
    
    # all keys in the $PATCH_NUMBER section are not required.
    
    F_log "Required file and fields check OK."
    return 0
}

F_rollback() {

    IS_MASTER=0
    sh /opt/trend/imss/script/is_master.sh
    if [ $? = 1 ] ; then
        IS_MASTER=1
    fi

    _value=`F_ini_get ${PATCHMGMINI} ${PATCH_NUMBER} "UIInterruption"`
    if [ ! -z "$_value" ] ; then
    if [ $_value = "1" ] ; then
    if [ $IS_MASTER = 1 ] ; then
        /opt/trend/imss/PostgreSQL/bin/psql -U sa -d imss -c "update tb_patch_status set status=4 where status=2;" > /dev/null 2>&1
        sleep 10
        /opt/trend/imss/PostgreSQL/bin/psql -U sa -d imss -c "update tb_patch_status set status=2 where status=4;" > /dev/null 2>&1
    fi
    fi
    fi
    
    #handle db rollback
    . ${TEMP_APPLY_DB_SH} "rollback" ${PATCH_FOLDER} ${PATCH_NUMBER}
    . ${TEMP_APPLY_DB_SH} "rollbackdbschema" ${PATCH_FOLDER} ${PATCH_NUMBER}
        if [ $? -ne 0 ];then
        
            F_log "Cannot rollback database, stop rollback hot fix."
            return 1
        fi
    _value=`F_ini_get ${PATCHMGMINI} ${PATCH_NUMBER} "CmdBeforeUpdate"`
    if [ ! -z "$_value" ] ; then
        F_log "Calling ${_value}"
        sh -c "${_value}"
    fi
    # SEGTT-276083, delete cron job if delete tag exists.
    if [ -e ${TAG_DELETE_VACUUM_CRON} ] ; then
        crontab -u root -l | grep -P -v 'db_maintain.sh\s+vacuumanalyzewholedb' > ${TEMP_CRONTAB}
        crontab -u root ${TEMP_CRONTAB}
        rm -rf ${TEMP_CRONTAB}
        rm -rf ${TAG_DELETE_VACUUM_CRON}
    fi 
    # no need to check return value of "rm -rf"
    # if it's failed in the middle, better keep rollbacking...
    pushd .
    cd /
    rm -rf `cat ${NORMAL_BACKUP_FILELIST}`
    popd
    
    if [ $NO_BACKUP_FILE -eq 0 ] ; then
        tar -C / -zxf ${NORMAL_BACKUP_TAR_GZ}
        if [ $? -ne 0 ] ; then
            F_log "Cannot untar files."
            ERROR_NUMBER=${ERR_UNTAR_BACKUP_FILES}
            
            # although this probably won't work at this time...
            _value=`F_ini_get ${PATCHMGMINI} ${PATCH_NUMBER} "CmdAfterUpdate"`
            if [ ! -z "$_value" ] ; then
                F_log "Calling ${_value}"
                sh -c "${_value}"
            fi
            return 1
        fi  
    fi
    
    # must write result before call CmdAfterUpdate
    # otherwise, imssmgr may not able to find the result during startup
    # for rollback, result_build and patch_title are useless
    F_write_result "1" "2" "0" "0" ${ERR_MSG_SUCCESS} "result_build" "patch_title"

    # Clear cache folder adminUI/ROOT/work and euqUI/ROOT/work
    rm -rf ${IMSS_HOME}/UI/adminUI/ROOT/work/
    rm -rf ${IMSS_HOME}/UI/euqUI/ROOT/work/
    
    _value=`F_ini_get ${PATCHMGMINI} ${PATCH_NUMBER} "CmdAfterUpdate"`
    if [ ! -z "$_value" ] ; then
        F_log "Calling ${_value}"
        sh -c "${_value}"
    fi
    
    # update patch.ini
    
    F_ini_delete_section ${PATCHMGMINI} ${PATCH_NUMBER}
    F_ini_delete_key ${PATCHMGMINI} "Patches" "Rollbackable"
    _applied_patches=`F_ini_get $PATCHMGMINI "Patches" "Applied"`
    _new_applied_patches=" "
    
    for i in $_applied_patches 
    do
        if [ "$PATCH_NUMBER" != "$i" ] ; then
            _new_applied_patches="${_new_applied_patches} $i"
        fi
    done
    
    F_ini_set $PATCHMGMINI "Patches" "Applied" "${_new_applied_patches}"
    
    # remove backup files
    
    rm -f ${NORMAL_ROLLBACK_SH}
    rm -f ${NORMAL_INI_UTIL_SH}
    rm -f ${NORMAL_PATCH_COMMON_SH}
    rm -f ${NORMAL_BACKUP_TAR_GZ}
    rm -f ${NORMAL_BACKUP_FILELIST}
    
    return 0
}


F_main() {

    F_log "Start rollback patch."
    
    if [ -f "$LOCK_FILE" ] ; then
        F_log "Another installing/uninstalling is in progress."
        ERROR_NUMBER=${ERR_ISINPROGRESS}
        return 1
    fi

    touch ${LOCK_FILE}
    
    if [ $? -ne 0 ] ; then
        F_log "Can not create lock file."
        ERROR_NUMBER=${ERR_FAILED_TO_LOCK}
        return 1
    fi
    
    # use size to indicate type, zero is update, one is rollback
    # this is used in IMSA 1, although IMSA 7 doesn't need this, just keep it.
    echo "r" > ${LOCK_FILE}
    
    F_check_all
    
    if [ $? -ne 0 ] ; then
        rm -f ${LOCK_FILE}
        return 1
    fi
        
    F_rollback

    if [ $? -ne 0 ] ; then
        rm -f ${LOCK_FILE}
        return 1
    fi
    
    # Success
    F_log "Done. Success"
    rm -f ${LOCK_FILE}

    rm -f /mnt/backup/patches/${PATCH_NUMBER} > /dev/null 2>&1

    cat /mnt/backup/patches/patches.history | grep -v "^${PATCH_NUMBER}$" > /mnt/backup/patches/patches.history.new
    rm -f /mnt/backup/patches/patches.history
    mv /mnt/backup/patches/patches.history.new /mnt/backup/patches/patches.history

    return 0
}

F_main
if [ $? -ne 0 ] ; then

    # for rollback, result_build and patch_title are useless
    F_write_result "1" "2" "0" "1" ${ERR_MSG_FAIL} "result_build" "patch_title"
    exit ${ERROR_NUMBER}
else
    exit 0
fi
